#
# For the year_ending array below, enter the "Year Ending:" dates listed in the calc attachment, you will find these
# dates down in the section where they list the Voucher Number, Liability Period, and the Date Voucher Due.
# That is all you need to do. Press the F5 key and copy the ouput to an empty file, print it, and compare the Due
# Dates to the calc attachment dates. Most likely, you will find one or two dates that will not agree because of
# a holiday. The form has comments about what to do if that happens.
#
year_ending = ["12/2016","01/2017","02/2017","03/2017","04/2017","05/2017","06/2017","07/2017","08/2017","09/2017","10/2017","11/2017"]

#
# Returns true if the year argument is a leap year, false if not. year is an integer.
#
def is_leap_year(year)
	if ((year % 4 == 0) and ((year % 100 != 0) or (year % 400 == 0)))
		return true
	else
		return false
	end
end

#
# NOTE: This method automatically computes dates for you. There is no need to change anything UNLESS
# the specs change, which of course, may happen. The only thing that will need to be changed (if at all) is
# when a computed date falls on a holiday, which changes from year to year. Just run the form and manually type in
# the info and see if the computed dates match the specs dates. If you see a difference, like the code generates
# a due date of 02/16/2009, BUT the specs (for 2006 in this example) say 02/17/2009, this would be because
# 02/16/2009 falls on the holiday President's Day. So you would need to go into the section that handles holidays
# and make the change. Simply delete the older year's holiday code from this method, replace with the current
# holiday dates. Check below, and you will see the handling for 02/16/2009. For 2006, only 02/16/2009 needed
# to be handled.
#
# This method computes the Liability Period and Coupon Due Dates delimited via the argument 'id'.
# 1 = Liability Due Date, 2 = Coupon Due Date. 'coupon' identifies the coupon number 1-4 with 5 flagging
# an extension coupon. Coupon due dates, for coupon numbers 1 - 4 are calculated to the 20th day of the 4th,
# 6th, 9th, and 12th month of the tax year delimited via arguments 'mon_end' and 'year_end'. Coupon
# Due Date for coupon number 5 is calculated to the 15th day of the 4th month following the close of the tax
# year. Liability Period Date (for coupon numbers 1 - 4) is calculated as the last day of the 3rd, 6th, 9th, and
# 12th month of the tax year. The Liability Period for coupon number 5 is calculated to the last day of the
# year ending date. Weekends are not allowed in the calculated dates, as well as holidays. A holiday list is
# maintained for checking purposes, this list will need to be updated for future years. All inputs are integer
# except for year_end which is then converted to an integer if the argument list checks out ok. Value returned
# is a string in MMDDYYYY format.
#
def compute_due_date(id, mon_end, year_end, coupon)
 
 coupon_intervals = [4, 6, 9, 12, 4]
 liability_intervals = [3, 6, 9, 12, 12]
 days_in_month = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31]
 
 #Check arguments first.
 return '00000000' unless (((1..2) === id) and ((1..12) === mon_end) and (year_end.length == 4) and ((1..5) === coupon)) 
 
 year_end = year_end.to_i
 
 #We will index into the interval arrays with this.
 coupon_adj = coupon - 1
 
 case id
  when 1
   #Computing Liability Due Date. These fall on the last day of the month.
   year_adj = year_end - 1
   mon_adj = mon_end + liability_intervals[coupon_adj]
   due_year = (mon_adj > 12 ? year_adj + 1 : year_adj)
   mod_mon = mon_adj % 12
   due_mon = ( mod_mon > 0 ? mod_mon : 12)
   due_day = if (due_mon == 2)
      (is_leap_year(due_year) ? 29 : 28)
     else
      days_in_month[due_mon - 1]
     end
  when 2
   #Computing Coupon Due Date.
   year_adj = (coupon < 5 ? year_end - 1 : year_end)
   mon_adj = mon_end + coupon_intervals[coupon_adj]
   due_year = (mon_adj > 12 ? year_adj + 1 : year_adj)
   mod_mon = mon_adj % 12
   due_mon = (mod_mon > 0 ? mod_mon : 12)
   due_day = (coupon < 5 ? 20 : 15)
   
   #Correct the Coupon due dates so that they do not fall on a weekend.
   #t.wday == 6 is Saturday, Sunday == 0.
   t = Time.mktime(due_year, due_mon, due_day)
   due_day += if (t.wday == 6) then 2
     elsif (t.wday == 0) then 1
     else 0 end
 end
 
 #Check for certain dates to conform to the specs. The holiday dates will
 #need to be adjusted in future years (or an algorithm could be written for it).
 #02/15/2010 is a holiday date, so we want 02/16/2010 for example.
 #~ if ((due_mon == 2) and (due_day == 15) and (due_year == 2010))
  #~ due_day = 16
 #~ end
 
 #Prepare due date for return - string with no '/'s.
 return ('%02.0f'% due_mon.to_s) + due_day.to_s + due_year.to_s 
end

year_ending.each { |year_end|

	p "Year Ending: #{year_end} :"
	p ""

	1.upto(5) { |coupon_num|

		#
		# Liability Period:
		#
		due_date = compute_due_date(1, year_end[0,2].to_i, year_end[3,4], coupon_num)
		due_date = "#{due_date[0,2]}/#{due_date[2,2]}/#{due_date[4,4]}"
		p "     Liability Period: #{due_date}"

		#
		# Date Coupon Due:
		#
		due_date =  compute_due_date(2, year_end[0,2].to_i, year_end[3,4], coupon_num)
		due_date = "#{due_date[0,2]}/#{due_date[2,2]}/#{due_date[4,4]}"
		p "     Date Coupon Due: #{due_date}"

		p ""
	}
}
